;################################################################################
;#										#
;# chipbasic v0.56								#
;# single chip basic computer with atmega 16					#
;# copyright (c) 2006 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU Lesser General Public			#
;# License as published by the Free Software Foundation; either			#
;# version 2 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# Lesser General Public License for more details.				#
;#										#
;# You should have received a copy of the GNU Lesser General Public		#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################
.set		ctype = 16			;mega16

.CSEG
.include	"m16def.inc"
.def		byte0 = r2
.def		byte1 = r3

.def		acc1l = r16
.def		acc1h = r17
.def		acc2l = r18
.def		acc2h = r19

.def		tempreg1 = r20
.def		tempreg2 = r21
.def		tempreg3 = r22
.def		tempreg4 = r23
.def		tempreg5 = r4
.def		tempreg6 = r5
.def		tempreg7 = r6
.def		tempreg8 = r7

.def		ereg = r24
.def		ctrl = r25

.equ		libmio_ram	= 0x060		;44 bytes libmio system RAM
.equ		libmio_sprites	= 0x08c		;20 bytes sprites
.equ		libmio_vram	= 0x0a0		;690 bytes video RAM (-0x352)
.equ		expar_ros	= 0x368		;22 bytes operand stack
.equ		varspace	= 0x368		;64 bytes basic variables / Array
.equ		bas_ram		= 0x3a8		;11 bytes basic system RAM
.equ		bas_stacks	= 0x3b3		;45 bytes basic stacks

.equ		bas_inbuf	= 0x3e0		;32 bytes input buffer
.equ		bas_linebuf	= 0x400		;32 bytes line buffer
.equ		stack		= 0x45f		;64 bytes system stack

.include	"libmio/definitions.asm"
.include	"macros.inc"

.set		bas_lines=20
.set		mem_linelen=25

.org 0x0000
		rjmp	start
.org OC1Baddr	
		libmio_vint

		
start:		cli
		ldi	XL,0x00
		mov	byte0,XL
		ldi	XL,0x01
		mov	byte1,XL
		ldi	XL,HIGH(stack)
		out	SPH,XL
		ldi	XL,LOW(stack)
		out	SPL,XL
	    	libmio_init
		call	i2c_initl
		libmio_clrscr
		clr	ereg
		sei
		lds	XL,libmio_config	;config
		sbrc	XL,4			;autorun pin
loop:		jmp	menu_main		;editor
loop1:		call	tbrun			;run program
		rjmp	loop1			;run again

.include	"modules/i2c.asm"
.include	"modules/menu.asm"
.include	"modules/editor.asm"
.include	"modules/exparser.asm"
.include	"modules/memory.asm"
.include	"modules/interpreter.asm"
.include	"modules/tokenizer.asm"


.include	"libs.inc"


.org	0x1600
libmio_table:
.include	"libmio/table.inc"
		